<?php
/**
 * Plugin Name: Block Bookings For Custom Order Status Customisation
 * Plugin URI: https://www.pluginhive.com/
 * Description: This plugin can be used to block bookings for custom order status along with the WooCommerce Bookings & Appointment plugin and WooCommerce Order Status Manager plugin.
 * Version: 1.0.2
 * Author: PluginHive
 * Author URI: https://pluginhive.com
 * WC tested up to: 5.6.0
 * Text Domain: ph-block-bookings-for-custom-order-status-customisation
 */

//  Ticket Number - 126219
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! defined( 'Block_Bookings_For_Custom_Order_Status_Customisation_PLUGIN_FILE' ) )
	define( 'Block_Bookings_For_Custom_Order_Status_Customisation_PLUGIN_FILE', __FILE__ );


function ph_block_bookings_for_custom_order_status(){

    if ( !is_plugin_active('ph-bookings-appointments-woocommerce-premium/ph-bookings-appointments-woocommerce-premium.php') ){
        deactivate_plugins( basename( __FILE__ ) );
        wp_die( __("Oops! You tried installing the Block Bookings For Custom Order Status Customisation Plugin without activating Bookings and Appointments For WooCommerce Premium Plugin. Kindly activate <b>Bookings and Appointments For WooCommerce Premium Plugin</b> and then try again", "bookings-and-appointments-for-woocommerce" ), "", array('back_link' => 1 ));
    }
}

register_activation_hook( __FILE__, 'ph_block_bookings_for_custom_order_status' );

//check if woocommerce exists
if ( !class_exists( 'woocommerce' ) || !class_exists('phive_booking_initialze_premium') ) {
    add_action( 'admin_init', 'Block_Bookings_For_Custom_Order_Status_Customisation_Admin_Init' );
    if ( ! function_exists( 'Block_Bookings_For_Custom_Order_Status_Customisation_Admin_Init' ) ) {
        function Block_Bookings_For_Custom_Order_Status_Customisation_Admin_Init() {
            if ( !class_exists( 'woocommerce' ) || !class_exists('phive_booking_initialze_premium') ){
                deactivate_plugins( plugin_basename( __FILE__ ) );
                wp_safe_redirect( admin_url('plugins.php') );
            }
        }
    }
}

if( ! class_exists('Block_Bookings_For_Custom_Order_Status_Customisation_Initialize') && in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && in_array( 'ph-bookings-appointments-woocommerce-premium/ph-bookings-appointments-woocommerce-premium.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) 
{
	class Block_Bookings_For_Custom_Order_Status_Customisation_Initialize 
    {

		/**
		 * Constructor
		 */
        public function __construct() 
        {
            add_filter('ph_bookings_all_bookings_query', array($this, 'ph_bookings_all_bookings_query_modify'), 10, 3);

            add_filter('ph_booking_query_for_bookings_count', array($this, 'ph_booking_query_for_bookings_count_modify'), 10, 2);
            add_filter('ph_booking_query_for_get_bookings_list', array($this, 'ph_booking_query_for_get_bookings_list_modify'), 10, 2);

            add_action('woocommerce_order_status_pickup', array( $this, 'phive_change_booking_status_based_on_order_details_custom' ), 5 );
            add_action('woocommerce_order_status_checked-out', array( $this, 'phive_change_booking_status_based_on_order_details_custom' ), 5 );
            add_action('woocommerce_order_status_returned', array( $this, 'phive_change_booking_status_based_on_order_details_custom' ), 5 );
            add_action('woocommerce_order_status_closed', array( $this, 'phive_change_booking_status_based_on_order_details_custom' ), 5 );
            add_action('woocommerce_order_status_past-due', array( $this, 'phive_change_booking_status_based_on_order_details_custom' ), 5 );

            // Admin Calendar View
            add_filter('ph_modify_admin_calendar_view_base_query', array($this, 'ph_modify_admin_calendar_view_base_query_func'), 10, 1);

            // Reminder Email
            add_filter('ph_modify_reminder_email_base_query', array($this, 'ph_modify_reminder_email_base_query_func'), 10, 1);

            // 96421
            add_filter('ph_modify_new_availability_calculation_query_where', array($this, 'ph_modify_new_availability_calculation_query_where'), 10, 7);

            // 96421
            add_filter('ph_modify_new_availability_calculation_query_where_assets', array($this, 'ph_modify_new_availability_calculation_query_where_assets'), 10, 6);

        }

        public function ph_modify_new_availability_calculation_query_where($where, $date, $product_id='', $asset_id='', $ignore_freezed=false, $calendar_for='', $settings=array())
        {
            $where = " AND woocommerce_order_status != 'closed'";
            return $where;
        }

        public function ph_modify_new_availability_calculation_query_where_assets($where, $asset_id, $current_time ,$ignore_freezed, $product_id, $settings)
        {
            $where = " AND woocommerce_order_status != 'closed'";
            return $where;
        }

        public function ph_modify_admin_calendar_view_base_query_func($query='')
        {
            global $wpdb;
            $query = "SELECT oitems.order_id, oitems.order_item_id,tr.object_id product_id, ometa.customer_name, ometa.customer_last_name, ometa.customer_billing_email, imeta.BookingStatus, imeta.BookFrom, imeta.BookTo, imeta.IntervalDetails, imeta.no_of_persons, imeta.is_ph_canceled
            FROM {$wpdb->prefix}posts
            INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
            INNER JOIN (
                    SELECT
                    order_item_id,
                    MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
                    MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
                    MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
                    MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
                    MAX(CASE WHEN meta_key = 'Number of persons' THEN meta_value Else '' END) AS no_of_persons,
                    MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails,
                    MAX(CASE WHEN meta_key = 'canceled' THEN meta_value ELSE '' END) AS is_ph_canceled
                    FROM {$wpdb->prefix}woocommerce_order_itemmeta
                    GROUP BY order_item_id
            ) as imeta on  imeta.order_item_id = oitems.order_item_id
            INNER JOIN (
                    SELECT
                    post_id,
                    MAX(CASE WHEN meta_key = '_billing_first_name' THEN meta_value ELSE '' END) AS customer_name,
                    MAX(CASE WHEN meta_key = '_billing_last_name' THEN meta_value ELSE '' END) AS customer_last_name,
                    MAX(CASE WHEN meta_key = '_billing_email' THEN meta_value ELSE '' END) AS customer_billing_email
                    FROM {$wpdb->prefix}postmeta
                    GROUP BY post_id
            ) as ometa on ometa.post_id = {$wpdb->prefix}posts.ID
            INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId
            INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
            INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
            WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
            AND (
                {$wpdb->prefix}posts.post_status = 'wc-pending'
                OR {$wpdb->prefix}posts.post_status = 'wc-processing'
                OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
                OR {$wpdb->prefix}posts.post_status = 'wc-completed'
                OR {$wpdb->prefix}posts.post_status = 'wc-cancelled'
                OR {$wpdb->prefix}posts.post_status = 'wc-refunded'
                OR {$wpdb->prefix}posts.post_status = 'wc-failed'
                OR {$wpdb->prefix}posts.post_status = 'wc-partially-paid'
                OR {$wpdb->prefix}posts.post_status = 'wc-partial-payment'
                OR {$wpdb->prefix}posts.post_status = 'wc-ng-complete'
                OR {$wpdb->prefix}posts.post_status = 'wc-pickup'
                OR {$wpdb->prefix}posts.post_status = 'wc-checked-out'
                OR {$wpdb->prefix}posts.post_status = 'wc-returned'
                OR {$wpdb->prefix}posts.post_status = 'wc-past-due'
                OR {$wpdb->prefix}posts.post_status = 'wc-closed'
            )
            AND tt.taxonomy IN ('product_type')
            AND t.slug = 'phive_booking'";
            
            return $query;
        }

        public function ph_modify_reminder_email_base_query_func($query = '')
        {
            global $wpdb;
			$query = "SELECT oitems.order_id, oitems.order_item_id,tr.object_id product_id,ometa.customer_name, ometa.billing_email, imeta.BookingStatus, imeta.BookFrom, imeta.BookTo, imeta.IntervalDetails,imeta.ReminderEmailStatus
			FROM {$wpdb->prefix}posts
			INNER JOIN (
					SELECT
					post_id,
					MAX(CASE WHEN meta_key = '_billing_email' THEN meta_value ELSE '' END) AS billing_email,
					MAX(CASE WHEN meta_key = '_billing_first_name' THEN meta_value ELSE '' END) AS customer_name
					FROM {$wpdb->prefix}postmeta
					GROUP BY post_id
			) as ometa on ometa.post_id = {$wpdb->prefix}posts.ID
			INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
			INNER JOIN (
					SELECT
					order_item_id,
					MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
					MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
					MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
					MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
					MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails,
					MAX(CASE WHEN meta_key = 'ReminderEmailStatus' THEN meta_value ELSE '' END) AS ReminderEmailStatus
					FROM {$wpdb->prefix}woocommerce_order_itemmeta
					GROUP BY order_item_id
			) as imeta on  imeta.order_item_id = oitems.order_item_id
			INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId
			INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
			INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
			WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
			AND (
				{$wpdb->prefix}posts.post_status = 'wc-pending'
				OR {$wpdb->prefix}posts.post_status = 'wc-processing'
				OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
				OR {$wpdb->prefix}posts.post_status = 'wc-completed'
				OR {$wpdb->prefix}posts.post_status = 'wc-partially-paid'
				OR {$wpdb->prefix}posts.post_status = 'wc-partial-payment'
				OR {$wpdb->prefix}posts.post_status = 'wc-ng-complete'
                OR {$wpdb->prefix}posts.post_status = 'wc-pickup'
                OR {$wpdb->prefix}posts.post_status = 'wc-checked-out'
                OR {$wpdb->prefix}posts.post_status = 'wc-returned'
                OR {$wpdb->prefix}posts.post_status = 'wc-past-due'
			)
			AND tt.taxonomy IN ('product_type')
			AND t.slug = 'phive_booking'";

            return $query;
        }

        public function ph_bookings_all_bookings_query_modify($query='', $product_id='', $ph_booking_placed_limit=1000)
        {
            if(!empty($product_id))
            {
                global $wpdb;
                //Query for getting booked dates
                $query = "SELECT oitems.order_id, oitems.order_item_id, tr.object_id product_id, imeta.BookingStatus, imeta.BookFrom, imeta.BookTo, imeta.IntervalDetails, imeta.no_of_persons, imeta.person_as_booking, imeta.buffer_before_id, imeta.buffer_after_id
                FROM {$wpdb->prefix}posts
                INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
                INNER JOIN (
                        SELECT
                        order_item_id,
                        MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
                        MAX(CASE WHEN meta_key = '_ph_booking_dlang_product_id' THEN meta_value ELSE '' END) AS DefaultLangProductId,
                        MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
                        MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
                        MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
                        MAX(CASE WHEN meta_key = 'buffer_before_id' THEN meta_value ELSE '' END) AS buffer_before_id,
                        MAX(CASE WHEN meta_key = 'buffer_after_id' THEN meta_value Else '' END) AS buffer_after_id,
                        MAX(CASE WHEN meta_key = 'person_as_booking' THEN meta_value ELSE '' END) AS person_as_booking,
                        MAX(CASE WHEN meta_key = 'Number of persons' THEN meta_value Else '' END) AS no_of_persons,
                        MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails
                        FROM {$wpdb->prefix}woocommerce_order_itemmeta
                        GROUP BY order_item_id
                ) as imeta on  imeta.order_item_id = oitems.order_item_id
                INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId
                INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
                INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
                WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
                AND (
                    {$wpdb->prefix}posts.post_status Like 'wc-pending'
                    OR {$wpdb->prefix}posts.post_status = 'wc-processing'
                    OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
                    OR {$wpdb->prefix}posts.post_status = 'wc-completed'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partially-paid'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partial-payment'
                    OR {$wpdb->prefix}posts.post_status = 'wc-ng-complete'
                    OR {$wpdb->prefix}posts.post_status = 'wc-pickup'
                    OR {$wpdb->prefix}posts.post_status = 'wc-checked-out'
                    OR {$wpdb->prefix}posts.post_status = 'wc-returned'
                    OR {$wpdb->prefix}posts.post_status = 'wc-past-due'
                )
                AND tt.taxonomy IN ('product_type')
                AND t.slug = 'phive_booking' 
                AND (imeta.ProductId = '$product_id' OR imeta.DefaultLangProductId = '$product_id')
                AND imeta.BookingStatus != 'a:1:{i:0;s:8:\"canceled\";}'
                AND imeta.BookingStatus != 'canceled'
                ORDER BY oitems.order_item_id  DESC LIMIT 0, $ph_booking_placed_limit";
            }
            return $query;
        }
        
        public function phive_change_booking_status_based_on_order_details_custom($order_id = '')
        {
            // error_log('inside');
            if(class_exists('phive_booking_checkout_decorator') && !empty($order_id))
            {
                $obj = new phive_booking_checkout_decorator();
                $obj->map_booking_status_with_order_status['pickup'] = 'paid';
                $obj->map_booking_status_with_order_status['checked-out'] = 'paid';
                $obj->map_booking_status_with_order_status['returned'] = 'paid';
                $obj->map_booking_status_with_order_status['closed'] = 'paid';
                $obj->map_booking_status_with_order_status['past-due'] = 'paid';

                // error_log('$obj->map_booking_status_with_order_status : '.print_r($obj->map_booking_status_with_order_status,1));
                // error_log('calling phive_change_booking_status_based_on_order_details');
                $obj->phive_change_booking_status_based_on_order_details($order_id);
            }
        }

        public function ph_booking_query_for_bookings_count_modify($query='',$filters='')
        {
            if(!empty($filters))
            {
                global $wpdb;

                $query = "SELECT count(distinct oitems.order_item_id)
                FROM {$wpdb->prefix}posts
                INNER JOIN {$wpdb->prefix}postmeta ometa on ometa.post_id = {$wpdb->prefix}posts.ID
                INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
                INNER JOIN (
                        SELECT 
                        order_item_id,
                        MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
                        MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
                        MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
                        MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
                        MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails
                    FROM {$wpdb->prefix}woocommerce_order_itemmeta 
                    GROUP BY order_item_id
                ) as imeta on imeta.order_item_id = oitems.order_item_id
                INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId 
                INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id 
                INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
                WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
                AND (
                    {$wpdb->prefix}posts.post_status = 'wc-pending'
                    OR {$wpdb->prefix}posts.post_status = 'wc-processing'
                    OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
                    OR {$wpdb->prefix}posts.post_status = 'wc-completed'
                    OR {$wpdb->prefix}posts.post_status = 'wc-cancelled'
                    OR {$wpdb->prefix}posts.post_status = 'wc-refunded'
                    OR {$wpdb->prefix}posts.post_status = 'wc-failed'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partially-paid'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partial-payment'
                    OR {$wpdb->prefix}posts.post_status = 'wc-ng-complete'
                    OR {$wpdb->prefix}posts.post_status = 'wc-pickup'
                    OR {$wpdb->prefix}posts.post_status = 'wc-checked-out'
                    OR {$wpdb->prefix}posts.post_status = 'wc-returned'
                    OR {$wpdb->prefix}posts.post_status = 'wc-past-due'
                    OR {$wpdb->prefix}posts.post_status = 'wc-closed'
                ) 
                AND ometa.meta_key in ('_billing_first_name','_billing_email')
                AND tt.taxonomy IN ('product_type')
                AND t.slug = 'phive_booking'";
                $sub_query = "
                IF ( LENGTH( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[7-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) ) = 7,
                    CONCAT( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:7:".'"'."', -1), '".'"'."', -2),'".'"'."',1),'-01'),
                SUBSTRING_INDEX( SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[10-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) )";
                if( !empty($filters['ph_booking_status']) ){
                    $booking_status_len = strlen($filters['ph_booking_status']);
                    $query .= " AND (SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookingStatus, 'i:0;s:".$booking_status_len.":".'"'."', -1), '".'"'."', 1) = '".$filters['ph_booking_status']."' OR imeta.BookingStatus = '".$filters['ph_booking_status']."' ) ";
                }
                if( !empty($filters['ph_filter_product_ids']) ){
                    $query .= "AND imeta.ProductId = '".$filters['ph_filter_product_ids']."'";
                }
                if( !empty($filters['ph_filter_from']) ){
                    $query .= " AND (DATE(".$sub_query.") >= '".$filters['ph_filter_from']."'";
                    $query .= " OR DATE(imeta.BookFrom) >= '".$filters['ph_filter_from']."')";
                }
                if( !empty($filters['ph_filter_to']) ){
                    $query .= " AND (DATE(".$sub_query.") <= '".$filters['ph_filter_to']."'";
                    $query .= " OR DATE(imeta.BookFrom) <= '".$filters['ph_filter_to']."')";
                }

                if( !empty($filters['ph_filter_end_from']) ){
                    $sub_query_for_booking_end = <<<EOD
 AND "{$filters['ph_filter_end_from']}" <= IF(
( NOT ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
(
IF(
	(imeta.BookTo = imeta.BookFrom AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month'),
	IF (
		SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
		DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
		DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
	),
	IF(
		NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
		(
			IF(
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
				DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
				DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
			)
		),
		SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 )
	)
)
),
(
IF(
	( ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
	(
		IF(
			NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
			(
				IF (  
					SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
					DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
					DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
				)
			),
			(
				IF(
					SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) > 1 ,
					IF (
						SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day',
						DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) DAY),
						DATE_ADD(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), '-01'), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MONTH)
					),
					SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
				)
			)
		)
	),
	SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
)
)
)
EOD;
                    $query.= $sub_query_for_booking_end;
                }


                if( !empty($filters['ph_filter_end_to']) ){
                    $filter_end_to_with_time = $filters['ph_filter_end_to']; 
                    $filter_end_to_with_time.= " 23:59"; 
                    $sub_query_for_booking_end = <<<EOD
 AND "{$filter_end_to_with_time}" >= IF(
	( NOT ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
	(
		IF(
			(imeta.BookTo = imeta.BookFrom AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month'),
			IF (
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
				DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
				DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
			),
			IF(
				NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
				(
					IF(
						SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
						DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
						DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
					)
				),
				IF(
					SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
					last_day(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ),"-27")),
					SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 )
				)
			)
		)
	),
	(
		IF(
			( ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
			(
				IF(
					NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
					(
						IF (  
							SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
							DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
							DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
						)
					),
					(
						IF(
							SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) > 1 ,
							IF (
								SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day',
								DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) DAY),
								DATE_ADD(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), '-01'), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MONTH)
							),
							SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
						)
					)
				)
			),
			IF(
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
				last_day(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ),"-27")),
				SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
			)

		)
	)
)
EOD;
                    $query.= $sub_query_for_booking_end;
                }
            }
            return $query;
        }

        public function ph_booking_query_for_get_bookings_list_modify($query='',$filters='')
        {
            if(!empty($filters))
            {
                global $wpdb;
                $query = "SELECT oitems.order_id, oitems.order_item_id,tr.object_id product_id, ometa.customer_name, ometa.customer_last_name, ometa.customer_billing_email, imeta.BookingStatus, imeta.BookFrom, imeta.BookTo, imeta.IntervalDetails, imeta.no_of_persons
                FROM {$wpdb->prefix}posts
                INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
                INNER JOIN (
                        SELECT
                        order_item_id,
                        MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
                        MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
                        MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
                        MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
                        MAX(CASE WHEN meta_key = 'Number of persons' THEN meta_value Else '' END) AS no_of_persons,
                        MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails
                        FROM {$wpdb->prefix}woocommerce_order_itemmeta
                        GROUP BY order_item_id
                ) as imeta on  imeta.order_item_id = oitems.order_item_id
                INNER JOIN (
                        SELECT
                        post_id,
                        MAX(CASE WHEN meta_key = '_billing_first_name' THEN meta_value ELSE '' END) AS customer_name,
                        MAX(CASE WHEN meta_key = '_billing_last_name' THEN meta_value ELSE '' END) AS customer_last_name,
                        MAX(CASE WHEN meta_key = '_billing_email' THEN meta_value ELSE '' END) AS customer_billing_email
                        FROM {$wpdb->prefix}postmeta
                        GROUP BY post_id
                ) as ometa on ometa.post_id = {$wpdb->prefix}posts.ID
                INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId
                INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
                INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
                WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
                AND (
                    {$wpdb->prefix}posts.post_status = 'wc-pending'
                    OR {$wpdb->prefix}posts.post_status = 'wc-processing'
                    OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
                    OR {$wpdb->prefix}posts.post_status = 'wc-completed'
                    OR {$wpdb->prefix}posts.post_status = 'wc-cancelled'
                    OR {$wpdb->prefix}posts.post_status = 'wc-refunded'
                    OR {$wpdb->prefix}posts.post_status = 'wc-failed'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partially-paid'
                    OR {$wpdb->prefix}posts.post_status = 'wc-partial-payment'
                    OR {$wpdb->prefix}posts.post_status = 'wc-ng-complete'
                    OR {$wpdb->prefix}posts.post_status = 'wc-pickup'
                    OR {$wpdb->prefix}posts.post_status = 'wc-checked-out'
                    OR {$wpdb->prefix}posts.post_status = 'wc-returned'
                    OR {$wpdb->prefix}posts.post_status = 'wc-past-due'
                    OR {$wpdb->prefix}posts.post_status = 'wc-closed'
                )
                AND tt.taxonomy IN ('product_type')
                AND t.slug = 'phive_booking'";
                    
                // AND ometa.meta_key IN ('_billing_first_name','_billing_email')
                // AND CASE WHEN '_billing_first_name' THEN ometa.meta_key IN('_billing_first_name') ElSE ometa.meta_key IN ('_billing_email') END
                $sub_query = "
                    IF( LENGTH( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[7-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) ) = 7,
                        CONCAT( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:7:".'"'."', -1), '".'"'."', -2),'".'"'."',1),'-01' ),
                    SUBSTRING_INDEX( SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[10-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) )";
                
                if( !empty($filters['ph_booking_status']) ){
                    $len = strlen($filters['ph_booking_status']);
                    $query .= " AND (SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookingStatus, 'i:0;s:".$len.":".'"'."', -1), '".'"'."', 1) = '".$filters['ph_booking_status']."' OR imeta.BookingStatus = '".$filters['ph_booking_status']."' ) ";
                }
                if( !empty($filters['ph_filter_product_ids']) ){
                    $query .= " AND imeta.ProductId = '".$filters['ph_filter_product_ids']."'";
                }
                if( !empty($filters['ph_filter_from']) ){
                    $query .= " AND ( DATE(".$sub_query.")  >= '".$filters['ph_filter_from']."')";
                    // $query .= " OR DATE(imeta.BookFrom) >= '".$filters['ph_filter_from']."')";
                }
                if( !empty($filters['ph_filter_to']) ){
                    $query .= " AND (DATE(".$sub_query.") <= '".$filters['ph_filter_to']."')";
                    // $query .= " OR DATE(imeta.BookFrom) <= '".$filters['ph_filter_to']."')";
                }

                if( !empty($filters['ph_filter_end_from']) ){
                    $sub_query_for_booking_end = <<<EOD
 AND "{$filters['ph_filter_end_from']}" <= IF(
( NOT ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
(
IF(
	(imeta.BookTo = imeta.BookFrom AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month'),
	IF (
		SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
		DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
		DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
	),
	IF(
		NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
		(
			IF(
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
				DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
				DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
			)
		),
		SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 )
	)
)
),
(
IF(
	( ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
	(
		IF(
			NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
			(
				IF (  
					SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
					DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
					DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
				)
			),
			(
				IF(
					SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) > 1 ,
					IF (
						SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day',
						DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) DAY),
						DATE_ADD(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), '-01'), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MONTH)        
					),
					SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
				)
			)
		)
	),
	SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
)
)
)
EOD;
                    $query.= $sub_query_for_booking_end;
                }


                if( !empty($filters['ph_filter_end_to']) ){
                    $filter_end_to_with_time = $filters['ph_filter_end_to']; 
                    $filter_end_to_with_time.= " 23:59"; 
                    $sub_query_for_booking_end = <<<EOD
 AND "{$filter_end_to_with_time}" >= IF(
	( NOT ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
	(
		IF(
			(imeta.BookTo = imeta.BookFrom AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month'),
			IF (
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
				DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
				DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
			),
			IF(
				NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
				(
					IF(
						SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
						DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
						DATE_ADD(REPLACE(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ), '/', '-' ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
					)
				),
				IF(
					SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
					last_day(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 ),"-27")),
					SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookTo ,'";', 1), '"', -1 )
				)
			)
		)
	),
	(
		IF(
			( ( ISNULL(imeta.BookTo) OR imeta.BookTo = '') AND NOT ( ISNULL(imeta.IntervalDetails) OR imeta.IntervalDetails = '') ),
			(
				IF(
					NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day' AND NOT SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
					(
						IF (  
							SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'hour',
							DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) HOUR),
							DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MINUTE)        
						)
					),
					(
						IF(
							SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) > 1 ,
							IF (
								SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'day',
								DATE_ADD(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) DAY),
								DATE_ADD(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ), '-01'), INTERVAL SUBSTRING_INDEX(SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -3), '"', 1) MONTH)
							),
							SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
						)
					)
				)
			),
			IF(
				SUBSTRING_INDEX (SUBSTRING_INDEX( imeta.IntervalDetails , ':"', -1), '"', 1 ) = 'month',
				last_day(CONCAT(SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 ),"-27")),
				SUBSTRING_INDEX( SUBSTRING_INDEX( imeta.BookFrom ,'";', 1), '"', -1 )
			)

		)
	)
)
EOD;
		            $query.= $sub_query_for_booking_end;
		        }
            }
            return $query;
        }
	}
    new Block_Bookings_For_Custom_Order_Status_Customisation_Initialize();
}